# coding:utf-8
import paramiko
import os
import time
from tools_hjh.Tools import remove_leading_space


class SSHConn:
    """ 维护一个基于ssh协议的linux连接 """

    def __init__(self, host, port, username, password, wait_time=1):
        """ 给入连接信息初始化该连接，wait_time是连上后等待时间（等待系统初始化一些可能存在的东西），单位秒 """
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.wait_time = wait_time
        
        self.client = paramiko.SSHClient()
        self.client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self.client.connect(host, port, username, password, timeout=3, allow_agent=False, look_for_keys=False)
        time.sleep(wait_time)
        self.shell = self.client.invoke_shell()

    def exec_command(self, cmd):
        """ 执行单一命令，返回结果 """
        cmd = remove_leading_space(cmd)
        stdout = self.client.exec_command(cmd)[1]
        return stdout.read().decode("UTF-8", errors='ignore').strip()

    def exec_script(self, script):
        """ 执行shell脚本，返回交互式shell内容 """
        mess = ''
        shell = self.client.invoke_shell()
        script = remove_leading_space(script)
        shell.send(script + '\n')
        shell.send('exit\n')
        while not shell.exit_status_ready():
            mess = mess + shell.recv(1024).decode('UTF-8', errors='ignore')
        shell.close()
        return mess
        
    def _download(self, src, dst):
        if not os.path.exists(os.path.dirname(dst)):
            os.makedirs(os.path.dirname(dst))
        sftp = paramiko.SFTPClient.from_transport(self.transport, 1024)
        filetype = self.exec_command("ls -ld " + src)[0]
        if filetype == '-':
            sftp.get(src, dst)
        elif filetype == 'd':
            for file in sftp.listdir(src):
                self._download(src + '/' + file, dst + '/' + file)
        sftp.close()

    def download(self, src, dst):
        """ 下载文件到本地，dst只能是一个目录，表示下载到该目录下 """
        if not os.path.exists(dst):
            os.makedirs(dst)
        sftp = paramiko.SFTPClient.from_transport(self.transport, 1024)
        filetype = self.exec_command("ls -ld " + src)[0]
        if filetype == '-':
            sftp.get(src, dst + '/' + src.split('/')[-1])
        elif filetype == 'd':
            self._download(src, dst)
        sftp.close()

    def close(self):
        try:
            self.shell.close()
        except:
            pass
        try:
            self.client.close()
        except:
            pass
        
    def __del__(self):
        self.close()
    
    def refresh(self):
        self.close()
        return SSHConn(self.host, self.port, self.username, self.password)



# coding:utf-8
from tools_hjh.HTTPRequest import HTTPRequest
from math import ceil
from tools_hjh.Tools import mkdir
from tools_hjh.ThreadPool import ThreadPool
import os

    
def main():
    pass


class HTTPTools():
    
    @staticmethod
    def get_page(url, headers=None, data=None, proxies=None, encoding='UTF-8', timeout=None, stream=True, allow_redirects=True, verify=False):
        req = HTTPRequest(url=url, headers=headers, data=data, proxies=proxies, encoding=encoding, timeout=timeout, stream=stream, allow_redirects=allow_redirects, verify=verify)
        text = req.get_text()
        req.close()
        return text
    
    @staticmethod
    def get_size(url, headers=None, data=None, proxies=None, encoding='UTF-8', timeout=None, stream=True, allow_redirects=True, verify=False):
        req = HTTPRequest(url=url, headers=headers, data=data, proxies=proxies, encoding=encoding, timeout=timeout, stream=stream, allow_redirects=allow_redirects, verify=verify)
        text = req.get_size()
        req.close()
        return text
    
    @staticmethod
    def get_status_code(url, headers=None, data=None, proxies=None, encoding='UTF-8', timeout=None, stream=True, allow_redirects=True, verify=False):
        req = HTTPRequest(url=url, headers=headers, data=data, proxies=proxies, encoding=encoding, timeout=timeout, stream=stream, allow_redirects=allow_redirects, verify=verify)
        text = req.get_status_code()
        req.close()
        return text

    @staticmethod
    def download(url, filepath, replace=False, partsize=1024 * 1024, threadnum=1, headers=None, data=None, proxies=None, encoding='UTF-8', timeout=None, stream=True, allow_redirects=True, verify=False):
        req = HTTPRequest(url=url, headers=headers, data=data, proxies=proxies, encoding=encoding, timeout=timeout, stream=stream, allow_redirects=allow_redirects, verify=verify)
        if threadnum == 1:
            return req.download(filepath=filepath, replace=replace)
        else:
            tp = ThreadPool(threadnum)
            filesize = req.get_size()
            partnum = ceil(filesize / partsize)
            filepath = filepath.replace('\\', '/')
            path = filepath.rsplit('/', 1)[0] + '/'
            name = filepath.split('/')[-1]
            tmp_path = path + name + '_tmp/'
            mkdir(tmp_path)
            downloaded_files = []
            for idx in range(partnum):
                begin = idx * partsize + 1
                if idx == partnum - 1:
                    end = filesize
                else:
                    end = begin + partsize - 1
                new_filepath = tmp_path + str(idx)
                downloaded_files.append(new_filepath)
                new_headers = headers.copy()
                new_headers['Range'] = f'bytes={begin}-{end}'
                tp.run(HTTPTools.download, (url, new_filepath, True, 1024 * 1024, 1, new_headers, data, proxies, encoding, timeout, stream, allow_redirects, verify))
            tp.wait()
            
            size = 0
            filepath = open(filepath, 'wb')
            for downloaded_file in downloaded_files:
                if os.path.exists(downloaded_file):
                    f = open(downloaded_file, "rb")
                    tsfileRead = f.read()
                size = size + filepath.write(tsfileRead)
                f.close()
            # rm(m3u8TmpFileDir)
            filepath.close()
            return size
        
    @staticmethod
    def N_m3u8DL_CLI(url, filepath, filename, theadnum=32, exe_path='N_m3u8DL-CLI_v3.0.2.exe'):
        cmd = exe_path + ' --workDir ' + filepath + ' --saveName ' + filename + ' --maxThreads ' + str(theadnum) + ' --enableDelAfterDone ' + url
        os.popen('chcp 65001')
        os.system(cmd)

    
if __name__ == '__main__':
    main()



from tools_hjh.DBConn import DBConn
from tools_hjh.DBConn import QueryResults
from tools_hjh.OracleTools import OracleTools
from tools_hjh.MemoryDB import MemoryDB

from tools_hjh.SSHConn import SSHConn

from tools_hjh.ThreadPool import ThreadPool

from tools_hjh.HTTPRequest import HTTPRequest
from tools_hjh.Chrome import Chrome
from tools_hjh.Chrome import ChromePool
from tools_hjh.HTTPTools import HTTPTools

from tools_hjh.Log import Log

import tools_hjh.Tools



# coding:utf-8
from tools_hjh.ThreadPool import ThreadPool
from tools_hjh.Tools import remove_leading_space


class DBConn:
    """ 维护一个关系型数据库连接池，目前支持oracle，pgsql，mysql，sqlite；支持简单的sql执行 """
    
    ORACLE = 'oracle'
    PGSQL = 'pgsql'
    MYSQL = 'mysql'
    SQLITE = 'sqlite'

    def __init__(self, dbtype, host=None, port=None, db=None, username=None, password=None, poolsize=2, encoding='UTF-8', lib_dir=None):
        """ 初始化连接池
                如果是sqlite，db这个参数是要显示给入的
                如果是oracle，db给入的是sid或是servername都是可以的 """
                
        self.dbtype = dbtype
        self.host = host
        self.port = port
        self.db = db
        self.username = username
        self.password = password
        self.poolsize = poolsize
        self.encoding = encoding
        self.lib_dir = lib_dir
        
        self.runtp = ThreadPool(1, save_result=True)
        self.dbpool = None
        
        self.config = {
            'host':self.host,
            'port':self.port,
            'database':self.db,
            'user':self.username,
            'password':self.password,
            'maxconnections':self.poolsize,  # 最大连接数
            'blocking':True,  # 连接数达到最大时，新连接是否可阻塞
            'reset':False
        }
        
        if self.dbtype == 'pgsql' or self.dbtype == 'mysql':
            from dbutils.pooled_db import PooledDB
        if self.dbtype == "pgsql":
            import psycopg2
            self.dbpool = PooledDB(psycopg2, **self.config)
        elif self.dbtype == "mysql":
            import pymysql
            self.dbpool = PooledDB(pymysql, **self.config)
        elif self.dbtype == "sqlite": 
            import sqlite3
            from dbutils.persistent_db import PersistentDB
            self.dbpool = PersistentDB(sqlite3, database=db)
        elif self.dbtype == "oracle":
            import cx_Oracle
            if lib_dir is not None:
                cx_Oracle.init_oracle_client(lib_dir=lib_dir)
            try:
                dsn = cx_Oracle.makedsn(host, port, service_name=db)
                self.dbpool = cx_Oracle.SessionPool(user=username,
                                                    password=password,
                                                    dsn=dsn,
                                                    max=poolsize,
                                                    increment=1,
                                                    encoding=encoding)
            except:
                dsn = cx_Oracle.makedsn(host, port, sid=db)
                self.dbpool = cx_Oracle.SessionPool(user=username,
                                                    password=password,
                                                    dsn=dsn,
                                                    max=poolsize,
                                                    increment=1,
                                                    encoding=encoding)
    
    def __run(self, sql, param=None):
        sql = remove_leading_space(sql)
        # 替换占位符
        if self.dbtype == 'pgsql' or self.dbtype == 'mysql':
            sql = sql.replace('?', '%s')
        elif self.dbtype == 'oracle':
            sql = sql.replace('?', ':1')            
        else:
            pass
        
        # 获取连接
        if self.dbtype == "oracle":
            conn = self.dbpool.acquire()
        else:
            conn = self.dbpool.connection()
            
        cur = conn.cursor()
        
        '''
        # 执行SQL文件
        if type(param) == str and os.path.exists(param):
            file = open(param)
            sqlStr = file.read()
            file.close()
            if self.dbtype == 'pgsql':
                cur.execute(sqlStr)
            elif self.dbtype == 'sqlite':
                cur.executescript(sqlStr)
            elif self.dbtype == 'mysql' or self.dbtype == 'oracle':
                pass  # 暂不支持
            conn.commit()
            returnMess = self
        '''
        # 执行非SELECT语句
        if not sql.lower().strip().startswith("select"):
            sql = sql.strip()
            if type(param) == list:
                cur.executemany(sql, param)
            elif type(param) == tuple:
                cur.execute(sql, param)
            elif param is None:
                cur.execute(sql)
            conn.commit()
            rownum = cur.rowcount
            cur.close()
            conn.close()
            return rownum
    
        # 执行SELECT语句，会一次性把结果全拿到内存里，数据量大慎用
        if sql.lower().strip().startswith("select"):
            sql = sql.strip()
            col = []
            if param is None:
                cur.execute(sql)
            elif type(param) == tuple:
                cur.execute(sql, param)
            for c in cur.description:
                col.append(c[0])
            rows = cur.fetchall()
            rows_new = []
            for row in rows:
                row_new = []
                for cell in row:
                    cell_new = str(cell)
                    row_new.append(cell_new)
                rows_new.append(row_new)
            qrs = QueryResults(tuple(col), rows_new)
            cur.close()
            conn.close()
            return qrs
        
    def run(self, sql, param=None, wait=False):
        """ 执行点什么
        sql中的占位符可以统一使用“?”
        wait为True则会等待当前正在执行的sql，有bug，暂不处理，自用规避"""
        if wait == True:
            tpnum = self.runtp.run(self.__run, (sql, param))
            self.runtp.wait()
            rs = self.runtp.result_map.pop(tpnum)
            return rs
        else:
            return self.__run(sql, param)
        
    def insert(self, table_name, rows, pks=[]):
        """ 往指定table_name中插入数据，rows是一个多个元组的列表，每个元组表示一组参数；或者是一个元组 """
        if type(rows) == list and len(rows) > 0:
            row = rows[0]
        elif type(rows) == tuple:
            row = rows
        elif len(rows) == 0:
            return 0
        
        sql2 = ''
        pk_num = 0
        if type(pks) == str:
            pk_num = 1
            sql2 = sql2 + pks + ' = ?'
        elif type(pks) == list or type(pks) == tuple:
            pk_num = len(pks)
            for pk in pks:
                sql2 = sql2 + pk + ' = ? and '
            sql2 = sql2.rstrip('and ')
            
        param_num = '?'
        for _ in range(len(row) - 1 - pk_num):
            param_num = param_num + ', ?'
            
        if self.dbtype == 'oracle':
            if len(pks) == 0:
                sql = 'insert into ' + table_name + ' select ' + param_num + ' from dual'
            else:
                sql = 'insert into ' + table_name + ' select ' + param_num + ' from dual where not exists(select 1 from ' + table_name + ' where ' + sql2 + ')'
        else:
            if len(pks) == 0:
                sql = 'insert into ' + table_name + ' select ' + param_num
            else:
                sql = 'insert into ' + table_name + ' select ' + param_num + ' where not exists(select 1 from ' + table_name + ' where ' + sql2 + ')'
        
        return self.run(sql, rows)
                
    def close(self):
        try:
            self.dbpool.close()
        except:
            pass
        finally:
            self.dbpool = None
    
    def __del__(self):
        self.close()
        

class QueryResults:

    def __init__(self, cols=(), rows=[]):
        self.cols = cols
        self.rows = rows

    def get_cols(self):
        return self.cols

    def get_rows(self):
        return self.rows
    
    def set_cols(self, cols):
        self.cols = cols
        
    def set_rows(self, rows):
        self.rows = rows



# coding:utf-8
import time
import threading
from uuid import uuid1


class ThreadPool():
    """ 维护一个线程池 """
    
    def __init__(self, size, save_result=False):
        self.size = size
        self.pool_status = [0]
        self.result_map = {}
        self.save_result = save_result
        
    def clear(self):
        self.pool_status = [0]
        self.result_map = {}

    def run(self, func, args, kwargs={}):
        """ 主线程命令当前线程池从空闲线程中取一个线程执行给入的方法，如果池满，则主线程等待 """
        if self.pool_status[0] < self.size:
            thread_id = uuid1()
            t = myThread(func, args=args, kwargs=kwargs, thread_id=thread_id, pool_status=self.pool_status, result_map=self.result_map, save_result=self.save_result)
            t.start()
            return thread_id
        else:
            while self.pool_status[0] >= self.size:
                time.sleep(0.2)
            return self.run(func, args, kwargs)

    def get_results(self):
        return self.result_map
    
    def get_result(self, num):
        return self.result_map[num]
    
    def clear_result(self):
        self.result_map = {}

    def wait(self):
        """ 主线程等待，直到线程池不存在活动线程 """
        while self.pool_status[0] > 0:
            time.sleep(0.2)


class myThread (threading.Thread):

    def __init__(self, func, args, kwargs, thread_id, pool_status, result_map, save_result):
        threading.Thread.__init__(self)
        self.func = func
        self.args = args
        self.kwargs = kwargs
        self.thread_id = thread_id
        self.pool_status = pool_status
        self.result_map = result_map
        self.save_result = save_result

    def run(self):
        self.pool_status[0] = self.pool_status[0] + 1
        result = self.func(*self.args, **self.kwargs)
        if self.save_result:
            self.result_map[self.thread_id] = result
        self.pool_status[0] = self.pool_status[0] - 1

